#include "Shape.h"

USING_NS_CC;
using namespace std;

RenderTexture* ShapeFrame::rt;
vector<ShapeFrame*> ShapeFrame::shapeFrames;

Shape* Shape::createWithVerts(Vec2 *verts, int vertsLen, float width, float height, Color4B color)
{
	Shape *ret = new (std::nothrow) Shape(verts, vertsLen, width, height, color);
	if (ret && ret->init())
	{
		ret->autorelease();
		return ret;
	}
	else
	{
		CC_SAFE_DELETE(ret);
		return nullptr;
	}
}

Shape* Shape::createWithPrecision(float width, float height, Color4B color, float borderWidth, Color4B borderColor, Rect cornerRadius)
{
	Shape *ret = new (std::nothrow) Shape(width, height, color, borderWidth, borderColor, cornerRadius);
	if (ret && ret->init())
	{
		ret->autorelease();
		return ret;
	}
	else
	{
		CC_SAFE_DELETE(ret);
		return nullptr;
	}
}

Shape* Shape::create(float width, float height, Color4B color, float borderWidth, Color4B borderColor, int cornerRadius)
{
	Shape *ret = new (std::nothrow) Shape(width, height, color, borderWidth, borderColor, Rect(cornerRadius, cornerRadius, cornerRadius, cornerRadius));
	if (ret && ret->init())
	{
		ret->autorelease();
		return ret;
	}
	else
	{
		CC_SAFE_DELETE(ret);
		return nullptr;
	}
}

Shape::Shape(float width, float height, Color4B color, float borderWidth, Color4B borderColor, Rect cornerRadius)
{
	this->verts = nullptr;
	this->vertsLen = 0;
	this->width = width;
	this->height = height;
	this->color = color;
	this->borderWidth = borderWidth;
	this->borderColor = borderColor;
	this->cornerRadius = cornerRadius;
}

Shape::Shape(Vec2 *verts, int vertsLen, float width, float height, Color4B color)
{
	this->verts = verts;
	this->vertsLen = vertsLen;
	this->width = width;
	this->height = height;
	this->color = color;
	this->borderWidth = 0;
	this->borderColor = Color4B::BLACK;
	this->cornerRadius = Rect::ZERO;
}

bool Shape::init()
{
	if (!Node::init())
	{
		return false;
	}
	
	string uuid = verts ? to_string((int)verts[0].x) + to_string((int)verts[0].y) + to_string(vertsLen) : to_string((int)width) + to_string((int)height)
		+ to_string((int)color.r)
		+ to_string((int)color.g)
		+ to_string((int)color.b)
		+ to_string((int)color.a)
		+ to_string((int)borderWidth)
		+ to_string((int)borderColor.r)
		+ to_string((int)borderColor.g)
		+ to_string((int)borderColor.b)
		+ to_string((int)borderColor.a)
		+ to_string((int)cornerRadius.origin.x)
		+ to_string((int)cornerRadius.origin.y)
		+ to_string((int)cornerRadius.size.width)
		+ to_string((int)cornerRadius.size.height);

	auto shapeFrame = ShapeFrame::getShapeFrame(uuid);

	if (!shapeFrame)
	{
		auto drawnode = DrawNode::create();

		int segments = cornerRadius.origin.x + cornerRadius.origin.y + cornerRadius.size.width + cornerRadius.size.height;

		if (verts)
		{
			drawnode->drawPolygon(verts, vertsLen, Color4F(Color3B(color), color.a > 0), borderWidth / 2, Color4F(Color3B(borderColor), borderColor.a > 0));
		}
		else if (segments > 0)
		{
			Vec2 *verts = new Vec2[segments + 8];
			int i = 0;

			segments = cornerRadius.origin.x + 1;
			for (int j = 0; j <= segments; j++)
			{
				verts[i++] = Vec2(-width / 2 + cornerRadius.origin.x + cornerRadius.origin.x * cos((j * 90 / segments + 180) * M_PI / 180), -height / 2 + cornerRadius.origin.x + cornerRadius.origin.x * sin((j * 90 / segments + 180) * M_PI / 180));
			}

			segments = cornerRadius.origin.y + 1;
			for (int j = 0; j <= segments; j++)
			{
				verts[i++] = Vec2(width / 2 - cornerRadius.origin.y + cornerRadius.origin.y * cos((j * 90 / segments + 270) * M_PI / 180), -height / 2 + cornerRadius.origin.y + cornerRadius.origin.y * sin((j * 90 / segments + 270) * M_PI / 180));
			}

			segments = cornerRadius.size.width + 1;
			for (int j = 0; j <= segments; j++)
			{
				verts[i++] = Vec2(width / 2 - cornerRadius.size.width + cornerRadius.size.width * cos(j * 90 / segments * M_PI / 180), height / 2 - cornerRadius.size.width + cornerRadius.size.width * sin(j * 90 / segments * M_PI / 180));
			}

			segments = cornerRadius.size.height + 1;
			for (int j = 0; j <= segments; j++)
			{
				verts[i++] = Vec2(-width / 2 + cornerRadius.size.height + cornerRadius.size.height * cos((j * 90 / segments + 90) * M_PI / 180), height / 2 - cornerRadius.size.height + cornerRadius.size.height * sin((j * 90 / segments + 90) * M_PI / 180));
			}

			drawnode->drawPolygon(verts, i, Color4F(Color3B(color), color.a > 0), borderWidth / 2, Color4F(Color3B(borderColor), borderColor.a > 0));

			delete[] verts;
		}
		else
		{
			Vec2 verts[4] = {
				Vec2(-width / 2, -height / 2),
				Vec2(-width / 2, height / 2),
				Vec2(width / 2, height / 2),
				Vec2(width / 2, -height / 2)
			};
			drawnode->drawPolygon(verts, 4, Color4F(Color3B(color), color.a > 0), borderWidth / 2, Color4F(Color3B(borderColor), borderColor.a > 0));
		}

		shapeFrame = ShapeFrame::createShapeFrame(uuid, drawnode, width + borderWidth, height + borderWidth);
	}
	
	this->setContentSize(Size(width, height));
	this->setCascadeColorEnabled(true);
	this->setCascadeOpacityEnabled(true);

	sprite = shapeFrame ? Sprite::createWithSpriteFrame(shapeFrame->getSpriteFrame()) : Sprite::create();
	if (color.a > 0) sprite->setOpacity(color.a);
	sprite->setFlippedY(true);
	this->addChild(sprite);

	return true;
}

void Shape::setGlobalZOrder(float globalZOrder)
{
	Node::setGlobalZOrder(globalZOrder);
	sprite->setGlobalZOrder(globalZOrder);
}

float Shape::getGlobalZOrder()
{
	return sprite->getGlobalZOrder();
}

ShapeFrame* ShapeFrame::createShapeFrame(string &uuid, DrawNode *node, float width, float height) {
	if (!rt)
	{
		auto scale = Director::getInstance()->getContentScaleFactor();
		rt = RenderTexture::create(4096 / scale, 4096 / scale);
		rt->getSprite()->getTexture()->setAliasTexParameters();
		rt->retain();
	}

	auto size = rt->getSprite()->getTexture()->getContentSize();
	auto rect = Rect(0, 0, width, height);

	for (float y = 0; y < size.height; y++)
	{
		for (float x = 0; x < size.width; x++)
		{
			if (x + width > size.width) continue;
			if (y + height > size.height) return nullptr;

			auto temp = Rect(x - 1, y - 1, width + 2, height + 2);

			for (auto shapeFrame : shapeFrames)
			{
				if (shapeFrame->getSpriteFrame()->getRect().intersectsRect(temp))
				{
					goto a;
				}
			}

			rect.origin.x = x;
			rect.origin.y = y;

			goto b;

		a:
			continue;
		}
	}

	return nullptr;

b:
	rt->begin();
#if CC_TARGET_PLATFORM == CC_PLATFORM_IOS
	node->setPosition(width / 2 + rect.origin.x, size.height - height / 2 - rect.origin.y);
#else
	node->setPosition(width / 2 + rect.origin.x, height / 2 + rect.origin.y);
#endif
	node->Node::visit();
	rt->end();

	auto spriteFrame = SpriteFrame::createWithTexture(rt->getSprite()->getTexture(), rect);
	spriteFrame->retain();

	auto shapeFrame = new ShapeFrame(uuid, spriteFrame);
	shapeFrames.push_back(shapeFrame);

	return shapeFrame;
}

ShapeFrame* ShapeFrame::getShapeFrame(string &uuid) {
	for (auto shapeFrame : shapeFrames)
	{
		if (shapeFrame->uuid.compare(uuid) == 0)
		{
			return shapeFrame;
		}
	}
	return nullptr;
}
