#ifndef __SHAPE_H__
#define __SHAPE_H__

#include "cocos2d.h"

class Shape : public cocos2d::Node
{
public:
	static Shape* createWithVerts(cocos2d::Vec2 *verts, int vertsLen, float width, float height, cocos2d::Color4B color);
	static Shape* createWithPrecision(float width, float height, cocos2d::Color4B color, float borderWidth = 0, cocos2d::Color4B borderColor = cocos2d::Color4B::BLACK, cocos2d::Rect cornerRadius = cocos2d::Rect::ZERO);
	static Shape* create(float width, float height, cocos2d::Color4B color, float borderWidth = 0, cocos2d::Color4B borderColor = cocos2d::Color4B::BLACK, int cornerRadius = 0);
	Shape(float width, float height, cocos2d::Color4B color, float borderWidth, cocos2d::Color4B borderColor, cocos2d::Rect cornerRadius);
	Shape(cocos2d::Vec2 *verts, int vertsLen, float width, float height, cocos2d::Color4B color);
	cocos2d::Vec2 *verts;
	int vertsLen;
	float width;
	float height;
	cocos2d::Color4B color;
	float borderWidth;
	cocos2d::Color4B borderColor;
	cocos2d::Rect cornerRadius;
	virtual bool init() override;
	virtual void setGlobalZOrder(float globalZOrder) override;
	virtual float getGlobalZOrder();
private:
	cocos2d::Sprite *sprite;
};

class ShapeFrame
{
public:
	static ShapeFrame* createShapeFrame(std::string &uuid, cocos2d::DrawNode *node, float width, float height);
	static ShapeFrame* getShapeFrame(std::string &uuid);
	static void release() {
		if (rt)
		{
			rt->release();
			rt = nullptr;
		}

		for (auto shapeFrame : shapeFrames)
		{
			shapeFrame->spriteFrame->release();
		}

		shapeFrames.clear();
	}
	cocos2d::SpriteFrame* getSpriteFrame() {
		return spriteFrame;
	}
	static void save() {
		rt->saveToFile("qzf.png");
	}
private:
	static cocos2d::RenderTexture* rt;
	static std::vector<ShapeFrame*> shapeFrames;
	ShapeFrame(std::string uuid, cocos2d::SpriteFrame *spriteFrame) : uuid(uuid), spriteFrame(spriteFrame) {
	}
	std::string uuid;
	cocos2d::SpriteFrame *spriteFrame;
};

#endif
